@@ -1,3 +1,4 @@
+from ast import Raise
 import copy
 import json
 import os
@@ -14,6 +15,7 @@ from databricks_cli.dbfs.api import DbfsApi
 from databricks_cli.runs.api import RunsApi
 from databricks_cli.sdk.api_client import ApiClient
 from feathr.constants import *
+from feathr.version import get_maven_artifact_fullname
 from feathr.spark_provider._abc import SparkJobLauncher
 from loguru import logger
 from requests.structures import CaseInsensitiveDict
@@ -68,8 +70,10 @@ class _FeathrDatabricksJobLauncher(SparkJobLauncher):
         """
         src_parse_result = urlparse(local_path_or_http_path)
         file_name = os.path.basename(local_path_or_http_path)
-        # returned paths for the uploaded file
-        returned_path = os.path.join(self.databricks_work_dir, file_name)
+        # returned paths for the uploaded file. Note that we cannot use os.path.join here, since in Windows system it will yield paths like this:
+        # dbfs:/feathrazure_cijob_snowflake_9_30_157692\auto_generated_derived_features.conf, where the path sep is mixed, and won't be able to be parsed by databricks.
+        # so we force the path to be Linux style here.
+        cloud_dest_path = self.databricks_work_dir + "/" + file_name
         if src_parse_result.scheme.startswith('http'):
             with urlopen(local_path_or_http_path) as f:
                 # use REST API to avoid local temp file
@@ -77,14 +81,14 @@ class _FeathrDatabricksJobLauncher(SparkJobLauncher):
                 files = {'file': data}
                 # for DBFS APIs, see: https://docs.microsoft.com/en-us/azure/databricks/dev-tools/api/latest/dbfs
                 r = requests.post(url=self.workspace_instance_url+'/api/2.0/dbfs/put',
-                                  headers=self.auth_headers, files=files,  data={'overwrite': 'true', 'path': returned_path})
+                                  headers=self.auth_headers, files=files,  data={'overwrite': 'true', 'path': cloud_dest_path})
                 logger.info('{} is downloaded and then uploaded to location: {}',
-                             local_path_or_http_path, returned_path)
+                             local_path_or_http_path, cloud_dest_path)
         elif src_parse_result.scheme.startswith('dbfs'):
             # passed a cloud path
             logger.info(
                 'Skip uploading file {} as the file starts with dbfs:/', local_path_or_http_path)
-            returned_path = local_path_or_http_path
+            cloud_dest_path = local_path_or_http_path
         elif src_parse_result.scheme.startswith(('wasb','s3','gs')):
             # if the path starts with a location that's not a local path
             logger.error("File {} cannot be downloaded. Please upload the file to dbfs manually.", local_path_or_http_path)
@@ -95,24 +99,29 @@ class _FeathrDatabricksJobLauncher(SparkJobLauncher):
                 logger.info("Uploading folder {}", local_path_or_http_path)
                 dest_paths = []
                 for item in Path(local_path_or_http_path).glob('**/*.conf'):
-                    returned_path = self.upload_local_file(item.resolve())
-                    dest_paths.extend([returned_path])
-                returned_path = ','.join(dest_paths)
+                    cloud_dest_path = self._upload_local_file_to_workspace(item.resolve())
+                    dest_paths.extend([cloud_dest_path])
+                cloud_dest_path = ','.join(dest_paths)
             else:
-                returned_path = self.upload_local_file(local_path_or_http_path)
-        return returned_path
+                cloud_dest_path = self._upload_local_file_to_workspace(local_path_or_http_path)
+        return cloud_dest_path
 
-    def upload_local_file(self, local_path: str) -> str:
+    def _upload_local_file_to_workspace(self, local_path: str) -> str:
         """
         Supports transferring file from a local path to cloud working storage.
         """
         file_name = os.path.basename(local_path)
-        # returned paths for the uploaded file
-        returned_path = os.path.join(self.databricks_work_dir, file_name)
+        # returned paths for the uploaded file. Note that we cannot use os.path.join here, since in Windows system it will yield paths like this:
+        # dbfs:/feathrazure_cijob_snowflake_9_30_157692\auto_generated_derived_features.conf, where the path sep is mixed, and won't be able to be parsed by databricks.
+        # so we force the path to be Linux style here.
+        cloud_dest_path = self.databricks_work_dir + "/" + file_name
         # `local_path_or_http_path` will be either string or PathLib object, so normalize it to string
         local_path = str(local_path)
-        DbfsApi(self.api_client).cp(recursive=True, overwrite=True, src=local_path, dst=returned_path)
-        return returned_path
+        try:
+            DbfsApi(self.api_client).cp(recursive=True, overwrite=True, src=local_path, dst=cloud_dest_path)
+        except RuntimeError as e:
+            raise RuntimeError(f"The source path: {local_path}, or the destination path: {cloud_dest_path}, is/are not valid.") from e
+        return cloud_dest_path
 
     def submit_feathr_job(self, job_name: str, main_jar_path: str,  main_class_name: str, arguments: List[str], python_files: List[str], reference_files_path: List[str] = [], job_tags: Dict[str, str] = None, configuration: Dict[str, str] = {}, properties: Dict[str, str] = {}):
         """
@@ -158,8 +167,8 @@ class _FeathrDatabricksJobLauncher(SparkJobLauncher):
 
         # the feathr main jar file is anyway needed regardless it's pyspark or scala spark
         if not main_jar_path:
-            logger.info(f"Main JAR file is not set, using default package '{FEATHR_MAVEN_ARTIFACT}' from Maven")
-            submission_params['libraries'][0]['maven'] = { "coordinates": FEATHR_MAVEN_ARTIFACT }
+            logger.info(f"Main JAR file is not set, using default package '{get_maven_artifact_fullname()}' from Maven")
+            submission_params['libraries'][0]['maven'] = { "coordinates": get_maven_artifact_fullname() }
         else:
             submission_params['libraries'][0]['jar'] = self.upload_or_get_cloud_path(main_jar_path)
         # see here for the submission parameter definition https://docs.microsoft.com/en-us/azure/databricks/dev-tools/api/2.0/jobs#--request-structure-6
@@ -265,3 +274,4 @@ class _FeathrDatabricksJobLauncher(SparkJobLauncher):
             raise RuntimeError('Currently only paths starting with dbfs is supported for downloading results from a databricks cluster. The path should start with \"dbfs:\" .')
 
         DbfsApi(self.api_client).cp(recursive=True, overwrite=True, src=result_path, dst=local_folder)
+
