@@ -161,17 +161,17 @@ def kafka_test_setup(config_path: str):
     return client
 
 def registry_test_setup(config_path: str):
-
-
-    # use a new project name every time to make sure all features are registered correctly
+    # Use a new project name every time to make sure all features are registered correctly
+    # Project name example: feathr_ci_registry_2022_09_24_01_02_30
     now = datetime.now()
-    os.environ["project_config__project_name"] =  ''.join(['feathr_ci_registry','_', str(now.minute), '_', str(now.second), '_', str(now.microsecond)]) 
+    os.environ["project_config__project_name"] = f'feathr_ci_registry_{str(now)[:19].replace(" ", "_").replace(":", "_").replace("-", "_")}'
 
     client = FeathrClient(config_path=config_path, project_registry_tag={"for_test_purpose":"true"})
     request_anchor, agg_anchor, derived_feature_list = generate_entities()
 
     client.build_features(anchor_list=[agg_anchor, request_anchor], derived_feature_list=derived_feature_list)
     return client
+
 def registry_test_setup_partially(config_path: str):
     """Register a partial of a project. Will call `generate_entities()` and register only the first anchor feature.
     """
@@ -378,4 +378,34 @@ def get_online_test_table_name(table_name: str):
     now = datetime.now()
     res_table = '_'.join([table_name, str(now.minute), str(now.second)])
     print("The online Redis table is", res_table)
-    return res_table
\ No newline at end of file
+    return res_table
+
+def time_partition_pattern_test_setup(config_path: str, data_source_path: str):
+    now = datetime.now()
+    # set workspace folder by time; make sure we don't have write conflict if there are many CI tests running
+    os.environ['SPARK_CONFIG__DATABRICKS__WORK_DIR'] = ''.join(['dbfs:/feathrazure_cijob','_', str(now.minute), '_', str(now.second), '_', str(now.microsecond)]) 
+    os.environ['SPARK_CONFIG__AZURE_SYNAPSE__WORKSPACE_DIR'] = ''.join(['abfss://feathrazuretest3fs@feathrazuretest3storage.dfs.core.windows.net/feathr_github_ci','_', str(now.minute), '_', str(now.second) ,'_', str(now.microsecond)]) 
+    client = FeathrClient(config_path=config_path)
+
+    batch_source = HdfsSource(name="testTimePartitionSource",
+                          path=data_source_path,
+                          time_partition_pattern="yyyy/MM/dd"
+                          )
+    key = TypedKey(key_column="key0",
+               key_column_type=ValueType.INT32)
+    agg_features = [
+    Feature(name="f_loc_avg_output",
+            key=[key],
+            feature_type=FLOAT,
+            transform="f_location_avg_fare"),
+    Feature(name="f_loc_max_output",
+            feature_type=FLOAT,
+            key=[key],
+            transform="f_location_max_fare"),
+    ]
+
+    agg_anchor = FeatureAnchor(name="testTimePartitionFeatures",
+                           source=batch_source,
+                           features=agg_features)
+    client.build_features(anchor_list=[agg_anchor])
+    return client
\ No newline at end of file
